# Hunter's Vip Mod Addon
#  
# Install instructions:
#       1. Install Mattie's EventScripts 2.0 plugin:
#           http://mattie.info/cs/
#
#       2. Copy and upload this script to:
#           <gamedir>/addons/eventscripts/vip_mod/
#
#       3. Add the following line somewhere in autoexec.cfg:
#           es_load vip_mod
#
################################################################################
#import EventScripts
import es
#import Libraries
import gamethread
import keyvalues
import playerlib
import popuplib
import time
import os
from sqlite3 import dbapi2 as sqlite
#import Psyco Compiler
import psyco
psyco.full()

# Addon Information
info = es.AddonInfo()
info.name = "Vip Mod"
info.version = "2.0.0"
info.author = "Hunter"
info.url = "http://addons.eventscripts.com/addons/user/289"
info.description = "Tracks the played time of each player on your server"
info.basename = "vip_mod"

hunter_vip_mod_ver      = info.version
hunter_vip_mod_text     = 'Hunters '+info.name+', www.sourceplugins.de, '+info.version+', ES 2.0.0.247+ (Python Version)'

# Rename Database
if os.path.exists(es.getAddonPath('vip_mod')+'/es_vip_mod_storage.sqldb'):
    os.rename(es.getAddonPath('vip_mod')+'/es_vip_mod_storage.sqldb', es.getAddonPath('vip_mod')+'/vip_mod.sqldb')

# Server Variables
vip_mod_savetime        = es.ServerVar('vip_mod_savetime', '30', 'How much days should inactive users be stored, before their settings get deleted? ( default = 30 )')
vip_mod_msg_connect     = es.ServerVar('vip_mod_msg_connect', '1', 'Should the scriptaddon annouce player time stats on connect? ( 1 = on | 0 = off )')
vip_mod_msg_disconnect  = es.ServerVar('vip_mod_msg_disconnect', '1', 'Should the scriptaddon annouce player time stats on disconnect? ( 1 = on | 0 = off )')
vip_mod_msg_teamchange  = es.ServerVar('vip_mod_msg_teamchange', '1', 'Should the scriptaddon annouce player time stats on teamchange? ( 1 = on | 0 = off )')
vip_mod_trackmode       = es.ServerVar('vip_mod_trackmode', '0', 'Should the scriptaddon count rounds or kills? ( 0 = rounds & kills | 1 = rounds | -1 = kills )')
vip_mod_usertags        = es.ServerVar('vip_mod_usertags', '0', 'Enable UserTags feature? ( 1 = on | 0 = off )')
vip_mod_usermodels      = es.ServerVar('vip_mod_usermodels', '0', 'Enable UserModels feature? ( 1 = on | 0 = off )')

# Global Variables
vip_mod_maplist         = file(str(es.ServerVar('eventscripts_gamedir')).replace('\\', '/')+'/maplist.txt').readlines()
vip_mod_players         = {}
vip_mod_popups          = {}

# KeyValues Object
vip_mod_kv              = keyvalues.KeyValues(name='vip_mod')

# SQLite Object
vip_mod_db              = sqlite.connect(es.getAddonPath('vip_mod')+'/vip_mod.sqldb')

# Player Object
class Player(object):
    def __init__(self, userid):
        self.data = {}
        self.temp = {}
        self.userid = int(userid)
        self.player = playerlib.getPlayer(self.userid)
        self.uniqueid = playerlib.uniqueid(self.userid, True)
        self.cursor = vip_mod_db.cursor()
        self.cursor.execute('SELECT * FROM PlayerList WHERE uniqueid=? LIMIT 1;', (self.uniqueid,))
        if self.cursor.fetchone():
            self.cursor.execute('UPDATE PlayerList SET name_last=?, time_last=? WHERE uniqueid=?;', (self.player.attributes['name'], int(_gettimestamp()), self.uniqueid))
        else:
            self.cursor.execute('REPLACE INTO PlayerList (uniqueid,name,name_last,time_first,time_last,points_total) VALUES (?,?,?,?,?,0);', (self.uniqueid, self.player.attributes['name'], self.player.attributes['name'], int(_gettimestamp()), int(_gettimestamp())))
        self.temp = {'session_total':self.points_total}
    def __getattr__(self, name):
        if name in ['name','name_last','time_first','time_last','points_total','id']:
            if not name in self.data:
                self.cursor.execute('SELECT %s FROM PlayerList WHERE uniqueid=? LIMIT 1;' % name, (self.uniqueid,))
                self.data[name] = self.cursor.fetchone()[0]
            return self.data[name]
        elif name in self.temp and name.startswith('session_'):
            return self.data[name.replace('session_', 'points_')]-self.temp[name]
        elif name in self.temp:
            return self.temp[name]
    def __setattr__(self, name, value):
        if name in ['name','name_last','time_first','time_last','points_total']:
            self.cursor.execute('UPDATE PlayerList SET %s=? WHERE uniqueid=?;' % name, (value, self.uniqueid))
            self.data[name] = value
        else:
            self.__dict__[name] = value

def load():
    public = es.ServerVar('hu_vm', info.version, info.name)
    public.makepublic()

    vip_mod_kv.load(es.getAddonPath('vip_mod')+'/vip_mod.txt')
    vip_mod_db.cursor().execute('CREATE TABLE IF NOT EXISTS PlayerList (id INTEGER DEFAULT 1 NOT NULL PRIMARY KEY,uniqueid TEXT  UNIQUE NOT NULL,name TEXT  NOT NULL,name_last TEXT  NOT NULL,time_first INTEGER  NOT NULL,time_last INTEGER  NOT NULL,points_total INTEGER  NULL);')
    vip_mod_db.commit()

    es.regsaycmd('!vip', 'vip_mod/vipcmd', 'Vip Mod Stats')
    es.regsaycmd('!vipstats', 'vip_mod/saycmd', 'Vip Mod Stats')
    es.log(hunter_vip_mod_text)
    es.msg('#multi', '#green[VipMod] #defaultLoaded')

def unload():
    for popup in vip_mod_popups:
        vip_mod_popups[popup].delete()

    vip_mod_db.commit()
    vip_mod_db.close()

    es.unregsaycmd('!vip')
    es.unregsaycmd('!vipstats')
    es.msg('#multi', '#green[VipMod] #defaultUnloaded')
    
def es_map_start(event_var):
    vip_mod_db.cursor().execute('CREATE TABLE IF NOT EXISTS PlayerList (id INTEGER DEFAULT 1 NOT NULL PRIMARY KEY,uniqueid TEXT  UNIQUE NOT NULL,name TEXT  NOT NULL,name_last TEXT  NOT NULL,time_first INTEGER  NOT NULL,time_last INTEGER  NOT NULL,points_total INTEGER  NULL);')
    vip_mod_db.cursor().execute('DELETE FROM PlayerList WHERE time_last<?;', (int(_gettimestamp())-(int(vip_mod_savetime)*86400),))
    vip_mod_db.commit()

def es_client_command(event_var):
    if (str(event_var['command']) == '!hunter_vip_mod_ver') or (str(event_var['command']) == '!hunter_all_ver'):
        es.cexec(int(event_var['userid']), 'echo '+hunter_vip_mod_text)

def player_activate(event_var):
    gamethread.delayed(30, es.cexec, (int(event_var['userid']), 'echo '+hunter_vip_mod_text))
    gamethread.delayed(15, es.tell, (int(event_var['userid']), '#multi', '#green[VipMod] #defaultSay \'!vipstats\' for VIP stats'))

    player = _player(event_var['userid'])

    if not int(player.points_total) and int(vip_mod_msg_connect):
        gamethread.delayed(10, es.msg, ('#multi', '#green[VipMod] #lightgreen%s#default will be tracked!' % player.player.attributes['name']))
    elif int(vip_mod_msg_connect):
        gamethread.delayed(20, es.msg, ('#multi', '#green[VipMod] #lightgreen%s #defaultis #green%s #defaultwith VIP points: Total %s' % (player.player.attributes['name'], _checkrank(int(event_var['userid'])), player.points_total)))
    if int(vip_mod_usertags):
        gamethread.delayed(60, _checkname, (event_var['userid'], True))

def player_disconnect(event_var):
    if int(event_var['userid']) in vip_mod_players:
        if int(vip_mod_msg_disconnect):
            es.msg('#multi', '#green[VipMod] #lightgreen%s #defaultis #green%s #defaultwith VIP points: Total %s - Session %s' % (event_var['es_username'], _checkrank(int(event_var['userid'])), _gettotalpoints(event_var['userid']), _gettotalpoints(event_var['userid'], True)))
        del vip_mod_players[int(event_var['userid'])]

    vip_mod_db.commit()

def player_spawn(event_var):
    if not int(event_var['userid']) in vip_mod_players: return

    if int(vip_mod_usermodels):
        gamethread.queue(_checkmodel, (event_var['userid']))

def player_death(event_var):
    if event_var['attacker'] and event_var['es_attackerteam'] and int(event_var['userid']) != int(event_var['attacker']) and int(event_var['es_userteam']) != int(event_var['es_attackerteam']):
        if int(vip_mod_trackmode) <= 0:
            _givepoint(event_var['userid'])

    vip_mod_db.commit()

def player_team(event_var):
    if int(event_var['disconnect']): return

    if int(event_var['team']) != int(event_var['oldteam']) and int(event_var['oldteam']) > 1:
        if int(vip_mod_msg_teamchange):
            es.msg('#multi', '#green[VipMod] #lightgreen%s #defaultis #green%s #defaultwith VIP points: Total %s - Session %s' % (event_var['es_username'], _checkrank(int(event_var['userid'])), _gettotalpoints(event_var['userid']), _gettotalpoints(event_var['userid'], True)))

def player_changename(event_var):
    player = _player(event_var['userid'])
    player.name_last = player.player.attributes['name']

    if int(vip_mod_usertags):
        gamethread.delayed(15, _checkname, (event_var['userid']))

def round_start(event_var):
    if int(vip_mod_trackmode) >= 0:
        for userid in es.getUseridList():
            _givepoint(userid)

def round_end(event_var):
    vip_mod_db.commit()

def saycmd():
    vip_mod_db.commit()

    if es.getcmduserid() in es.getUseridList():
        arg1 = str(es.getargv(1))
        arg2 = str(es.getargv(2))
        if arg1.lower().startswith('top'):
            toplist = vip_mod_db.cursor()
            toplist.execute('SELECT id,name_last,uniqueid FROM PlayerList ORDER BY points_total DESC LIMIT 50;')
            toplist = toplist.fetchall()
            topmenu = popuplib.easymenu('vip_mod_list_%s' % es.getcmduserid(),None,_player_select)
            topmenu.settitle('Vip Mod Players')
            for player in toplist:
                if not player[2].startswith('BOT'):
                    topmenu.addoption(player[0], player[1])
            if toplist and topmenu:
                topmenu.send(es.getcmduserid())
                vip_mod_popups['list_%s'%es.getcmduserid()] = topmenu
        elif arg1.lower() == 'help':
            es.tell(es.getcmduserid(), '#multi', '#green[VipMod] #defaultType \'!vipstats [name]\' for personal stats')
            es.tell(es.getcmduserid(), '#multi', '#green[VipMod] #defaultType \'!vipstats top\' for top stats')
        elif len(arg1) and es.getuserid(arg1) > 0:
            _player_select(es.getcmduserid(), _player(es.getuserid(arg1)).id)
        else:
            _player_select(es.getcmduserid(), _player(es.getcmduserid()).id)

def _player_select(userid, choice, popupid=None):
    count = vip_mod_db.cursor()
    count.execute('SELECT COUNT(id) FROM PlayerList;')
    count = count.fetchone()[0]

    player = vip_mod_db.cursor()
    player.execute('SELECT id,name,name_last,points_total FROM PlayerList WHERE id=? LIMIT 1;', (choice,))
    player = player.fetchone()
    
    if player:
        popup = popuplib.create('vip_mod_menu_%s' % userid)
        popup.addline('VIP Details for %s' % player[2])
        if player[1] != player[2]:
            popup.addline('    aka %s' % player[1])
        popup.addline('------------------')
        popup.addline('->1. Points')
        popup.addline('    Total: %s' % player[3])
        if es.getuserid(player[2]):
            popup.addline('    Session: %s' % _player(es.getuserid(player[2])).session_total)
        nxttime = None
        nxttext = None
        for tagtime in sorted(vip_mod_kv['vipranks'], key=_keyintstr, reverse=True):
            tagtext = vip_mod_kv['vipranks'][str(tagtime)]
            if int(str(tagtime)) <= player[3]:
                popup.addline('    Current: %s' % tagtext)
                if nxttime and nxttext:
                    popup.addline('    Next: %s' % nxttext)
                    popup.addline('    Points: %s left' % (int(str(nxttime))-int(player[3])))
                else:
                    popup.addline('    Next: n/a')
                popup.addline('------------------')
                break
            nxttime = tagtime
            nxttext = tagtext
        if nxttime and nxttext and nxttime == tagtime:
            popup.addline('    Next: %s' % nxttext)
            popup.addline('    Points: %s left' % (int(str(nxttime))-int(player[3])))
            popup.addline('------------------')
        if int(vip_mod_usertags):
            nxttime = None
            nxttext = None
            for tagtime in sorted(vip_mod_kv['usertags'], key=_keyintstr, reverse=True):
                tagtext = vip_mod_kv['usertags'][str(tagtime)]
                if int(str(tagtime)) <= player[3]:
                    popup.addline('    Current: %s' % tagtext.replace('%', ''))
                    if nxttime and nxttext:
                        popup.addline('    Next: %s' % nxttext.replace('%', ''))
                        popup.addline('    Points: %s left' % (int(str(nxttime))-int(player[3])))
                    else:
                        popup.addline('    Next: n/a')
                    popup.addline('------------------')
                    break
                nxttime = tagtime
                nxttext = tagtext
            if nxttime and nxttext and nxttime == tagtime:
                popup.addline('    Next: %s' % nxttext.replace('%', ''))
                popup.addline('    Points: %s left' % (int(str(nxttime))-int(player[3])))
                popup.addline('------------------')
        if int(vip_mod_usermodels):
            nxttime = None
            nxttext = None
            for modeltime in sorted(vip_mod_kv['usertags'], key=_keyintstr, reverse=True):
                modeltext = vip_mod_kv['usertags'][str(modeltime)]
                if int(str(modeltime)) <= player[3]:
                    popup.addline('    Current: %s' % os.path.basename(modeltext))
                    if nxttime and nxttext:
                        popup.addline('    Next: %s' % os.path.basename(nxttext))
                        popup.addline('    Points: %s left' % (int(str(nxttime))-int(player[3])))
                    else:
                        popup.addline('    Next: n/a')
                    popup.addline('------------------')
                    break
                nxttime = modeltime
                nxttext = modeltext
            if nxttime and nxttext and nxttime == tagtime:
                popup.addline('    Next: %s' % os.path.basename(nxttext))
                popup.addline('    Points: %s left' % (int(str(nxttime))-int(player[3])))
                popup.addline('------------------')
        popup.addline('->8. Calculated in Realtime')
        popup.addline('->9. Tracking %s Players' % count)
        popup.addline('------------------')
        if popupid and popuplib.exists(popupid):
            popup.addline('->0. Back')
            popup.submenu(10, popuplib.find(popupid))
        else:
            popup.addline('->0. Close')
        popup.send(userid)
        vip_mod_popups['menu_%s'%userid] = popup
        
def vipcmd():
    userid = es.getcmduserid()

    count = vip_mod_db.cursor()
    count.execute('SELECT COUNT(id) FROM PlayerList;')
    count = count.fetchone()[0]

    player = vip_mod_db.cursor()
    player.execute('SELECT id,name,name_last,points_total FROM PlayerList WHERE id=? LIMIT 1;', (_player(userid).id,))
    player = player.fetchone()

    if player:
        popup = popuplib.create('vip_mod_comm_%s' % userid)
        popup.addline('VIP Menu for %s' % player[2])
        if player[1] != player[2]:
            popup.addline('    aka %s' % player[1])
        popup.addline('------------------')
        popup.addline('->1. Points')
        popup.addline('    Total: %s' % player[3])
        if es.getuserid(player[2]):
            popup.addline('    Session: %s' % _player(es.getuserid(player[2])).session_total)
        nxttime = None
        nxttext = None
        for tagtime in sorted(vip_mod_kv['vipranks'], key=_keyintstr, reverse=True):
            tagtext = vip_mod_kv['vipranks'][str(tagtime)]
            if int(str(tagtime)) <= player[3]:
                popup.addline('    Current: %s' % tagtext)
                if nxttime and nxttext:
                    popup.addline('    Next: %s' % nxttext)
                    popup.addline('    Points: %s left' % (int(str(nxttime))-int(player[3])))
                else:
                    popup.addline('    Next: n/a')
                popup.addline('------------------')
                break
            nxttime = tagtime
            nxttext = tagtext
        if nxttime and nxttext and nxttime == tagtime:
            popup.addline('    Next: %s' % nxttext)
            popup.addline('    Points: %s left' % (int(str(nxttime))-int(player[3])))
            popup.addline('------------------')
        for tagtime in sorted(vip_mod_kv['vipfeatures'], key=_keyintstr, reverse=True):
            tagtext = vip_mod_kv['vipfeatures'][str(tagtime)]
            if int(str(tagtime)) <= player[3]:
                if 'friendlyfire' in tagtext:
                    popup.addline('->3. FriendlyFire on/off')
                    popup.select(3, _friendlyfire_select)
                if 'nextmap' in tagtext:
                    popup.addline('->4. Nextmap')
                    popup.select(4, _nextmap_select)
                if 'changemap' in tagtext:
                    popup.addline('->5. Changemap')
                    popup.select(5, _changemap_select)
                if 'kick' in tagtext:
                    popup.addline('->6. Kick')
                    popup.select(6, _kick_select)
                if tagtext:
                    popup.addline('------------------')
                break
        popup.addline('->8. Calculated in Realtime')
        popup.addline('->9. Tracking %s Players' % count)
        popup.addline('------------------')
        popup.addline('->0. Close')
        popup.send(userid)
        vip_mod_popups['comm_%s'%userid] = popup

def _friendlyfire_select(userid, choice, popupid):
    if int(es.ServerVar('mp_friendlyfire')):
        es.ServerVar('mp_friendlyfire').set(0)
        es.msg('#multi', '#green[VipMod] #lightgreen%s #defaultas #green%s #lightgreendeactivated #greenFriendlyFire' % (event_var['es_username'], _checkrank(int(event_var['userid']))))
    else:
        es.ServerVar('mp_friendlyfire').set(1)
        es.msg('#multi', '#green[VipMod] #lightgreen%s #defaultas #green%s #lightgreenactivated #greenFriendlyFire' % (event_var['es_username'], _checkrank(int(event_var['userid']))))

def _nextmap_select(userid, choice, popupid):
    if popupid == 'vip_mod_maplist':
        es.server.cmd('xa_setnextmap %s' % choice)
        es.server.cmd('ma_setnextmap %s' % choice)
        es.server.cmd('admin_setnextmap %s' % choice)
        es.ServerVar('eventscripts_nextmapoverride').set(choice)
        es.msg('#multi', '#green[VipMod] #lightgreen%s #defaultas #green%s #defaulthas set the next map to #green%s' % (event_var['es_username'], _checkrank(int(event_var['userid'])), choice))
    else:
        popup = popuplib.create('vip_mod_maplist', None, _nextmap_select)
        popup.settitle('Vip Mod\nSelect a map')
        popup.vguititle = 'Vip Mod - Select a map'
        for mapname in vip_mod_maplist:
            popup.addoption(mapname, mapname)
        popup.send()
        vip_mod_popups['maplist'] = popup

def _changemap_select(userid, choice, popupid):
    if popupid == 'vip_mod_maplist':
        es.server.cmd('xa_setnextmap %s' % choice)
        es.server.cmd('ma_setnextmap %s' % choice)
        es.server.cmd('admin_setnextmap %s' % choice)
        es.server.cmd('es_xdelayed 3 changelevel %s' % choice)
        es.ServerVar('eventscripts_nextmapoverride').set(choice)
        es.msg('#multi', '#green[VipMod] #lightgreen%s #defaultas #green%s #defaultwill change the map to #green%s' % (event_var['es_username'], _checkrank(int(event_var['userid'])), choice))
    else:
        popup = popuplib.create('vip_mod_maplist', None, _changemap_select)
        popup.settitle('Vip Mod\nSelect a map')
        popup.vguititle = 'Vip Mod - Select a map'
        for mapname in vip_mod_maplist:
            popup.addoption(mapname, mapname)
        popup.send()
        vip_mod_popups['maplist'] = popup

def _kick_select(userid, choice, popupid):
    if popupid == 'vip_mod_playerlist':
        choice.kick('Kicked by VIP Player!')
        es.msg('#multi', '#green[VipMod] #lightgreen%s #defaultas #green%s #defaultkicked player #green%s' % (event_var['es_username'], _checkrank(int(event_var['userid'])), choice.attributes['name']))
    else:
        popup = popuplib.create('vip_mod_playerlist', None, _kick_select)
        popup.settitle('Vip Mod\nSelect a player')
        popup.vguititle = 'Vip Mod - Select a player'
        for player in playerlib.getPlayerList('#human'):
            popup.addoption(player, player.attributes['name'])
        popup.send()
        vip_mod_popups['playerlist'] = popup

def _player(userid):
    if not int(userid) in vip_mod_players:
        vip_mod_players[int(userid)] = Player(userid)
    try:
        vip_mod_players[int(userid)].player.refreshAttributes()
    except:
        pass
    return vip_mod_players[int(userid)]

def _givepoint(userid):
    player = _player(userid)
    player.points_total += 1

def _gettotalpoints(userid, session=False):
    player = _player(userid)
    if not session:
        return player.points_total
    else:
        return player.session_total

def _checkrank(userid):
    if int(userid) in es.getUseridList():
        player = _player(userid)
        if int(player.player.attributes['isbot']) == 0:
            for ranktime in sorted(vip_mod_kv['vipranks'], key=_keyintstr, reverse=True):
                ranktext = vip_mod_kv['vipranks'][str(ranktime)]
                if int(str(ranktime)) <= player.points_total:
                    return ranktext
    return 'Guest'

def _checkname(userid, join=False):
    if int(userid) in es.getUseridList():
        player = _player(userid)
        if int(player.player.attributes['isbot']) == 0:
            newname = player.player.attributes['name']
            for tagtime in vip_mod_kv['usertags']:
                tagtext = vip_mod_kv['usertags'][str(tagtime)]
                if int(str(tagtime)) > player.points_total and tagtext.replace('%') in player.player.attributes['name'] and not player.player.attributes['steamid'] in vip_mod_kv['usertags_idlist'] and not int(vip_mod_kv['usertags_idlist'][player.player.attributes['steamid']]) == 2:
                    newname = newname.replace(tagtext.replace('%'), '')
                    es.cexec(userid, 'name %s' % player.player.attributes['name'].replace(tagtext.replace('%'), ''))
                    es.tell(userid, '#multi', '#green[VipMod] #lightgreenYou are using a protected nickname tag! Remove it or you will stay blinded!')
                    player.player.set('flash', (255, 5))
                    gamethread.delayed(5, _checkname, (userid))
            if join:
                for tagtime in sorted(vip_mod_kv['usertags'], key=_keyintstr, reverse=True):
                    tagtext = vip_mod_kv['usertags'][str(tagtime)]
                    if int(str(tagtime)) <= player.points_total and not tagtext.replace('%') in player.player.attributes['name'] and not player.player.attributes['steamid'] in vip_mod_kv['usertags_idlist'] and not int(vip_mod_kv['usertags_idlist'][player.player.attributes['steamid']]) == 1:
                        es.cexec(userid, 'name %s' % tagtext.replace('%', newname))
                        es.tell(userid, '#multi', '#green[VipMod] #lightgreenYou are allowed to use a protected nickname tag: %s' % newname)
                        break

def _checkmodel(userid):
    if int(userid) in es.getUseridList():
        player = _player(userid)
        if int(player.player.attributes['isbot']) == 0:
            for modeltime in sorted(vip_mod_kv['usermodels'], key=_keyintstr, reverse=True):
                modeltext = vip_mod_kv['usermodels'][str(modeltime)]
                if int(str(modeltime)) <= player.points_total and not player.player.attributes['steamid'] in vip_mod_kv['usermodels_idlist'] and not int(vip_mod_kv['usermodels_idlist'][player.player.attributes['steamid']]) == 1:
                    player.player.set('model', modeltext)
                    break

def _keyintstr(key):
    return int(str(key))

def _gettimestamp():
    return time.time()-1136049334 # es.gettimestamp emulation
